@@ -27,9 +27,10 @@ namespace Kokkos {
 namespace Experimental {
 namespace Impl {
 
-template <class IndexType, class IteratorType1, class IteratorType2,
-          class ReducerType, class BinaryPredicateType>
+template <class IteratorType1, class IteratorType2, class ReducerType,
+          class BinaryPredicateType>
 struct StdMismatchRedFunctor {
+  using index_type     = typename IteratorType1::difference_type;
   using red_value_type = typename ReducerType::value_type;
 
   IteratorType1 m_first1;
@@ -38,14 +39,15 @@ struct StdMismatchRedFunctor {
   BinaryPredicateType m_predicate;
 
   KOKKOS_FUNCTION
-  void operator()(const IndexType i, red_value_type& red_value) const {
+  void operator()(const index_type i, red_value_type& red_value) const {
     const auto& my_value1 = m_first1[i];
     const auto& my_value2 = m_first2[i];
 
-    auto rv =
-        !m_predicate(my_value1, my_value2)
-            ? red_value_type{i}
-            : red_value_type{::Kokkos::reduction_identity<IndexType>::min()};
+    // FIXME_NVHPC using a ternary operator causes problems
+    red_value_type rv = {i};
+    if (m_predicate(my_value1, my_value2)) {
+      rv = {::Kokkos::reduction_identity<index_type>::min()};
+    }
 
     m_reducer.join(red_value, rv);
   }
@@ -59,9 +61,12 @@ struct StdMismatchRedFunctor {
         m_predicate(std::move(predicate)) {}
 };
 
+//
+// exespace impl
+//
 template <class ExecutionSpace, class IteratorType1, class IteratorType2,
           class BinaryPredicateType>
-::Kokkos::pair<IteratorType1, IteratorType2> mismatch_impl(
+::Kokkos::pair<IteratorType1, IteratorType2> mismatch_exespace_impl(
     const std::string& label, const ExecutionSpace& ex, IteratorType1 first1,
     IteratorType1 last1, IteratorType2 first2, IteratorType2 last2,
     BinaryPredicateType predicate) {
@@ -76,9 +81,6 @@ template <class ExecutionSpace, class IteratorType1, class IteratorType2,
   using index_type           = typename IteratorType1::difference_type;
   using reducer_type         = FirstLoc<index_type>;
   using reduction_value_type = typename reducer_type::value_type;
-  using functor_type =
-      StdMismatchRedFunctor<index_type, IteratorType1, IteratorType2,
-                            reducer_type, BinaryPredicateType>;
 
   // trivial case: note that this is important,
   // for OpenMPTarget, omitting special handling of
@@ -95,7 +97,9 @@ template <class ExecutionSpace, class IteratorType1, class IteratorType2,
   reducer_type reducer(red_result);
   ::Kokkos::parallel_reduce(
       label, RangePolicy<ExecutionSpace>(ex, 0, num_elemen_par_reduce),
-      functor_type(first1, first2, reducer, std::move(predicate)), reducer);
+      // use CTAD
+      StdMismatchRedFunctor(first1, first2, reducer, std::move(predicate)),
+      reducer);
 
   // fence not needed because reducing into scalar
 
@@ -118,13 +122,83 @@ template <class ExecutionSpace, class IteratorType1, class IteratorType2,
 }
 
 template <class ExecutionSpace, class IteratorType1, class IteratorType2>
-::Kokkos::pair<IteratorType1, IteratorType2> mismatch_impl(
+::Kokkos::pair<IteratorType1, IteratorType2> mismatch_exespace_impl(
     const std::string& label, const ExecutionSpace& ex, IteratorType1 first1,
     IteratorType1 last1, IteratorType2 first2, IteratorType2 last2) {
   using value_type1 = typename IteratorType1::value_type;
   using value_type2 = typename IteratorType2::value_type;
   using pred_t      = StdAlgoEqualBinaryPredicate<value_type1, value_type2>;
-  return mismatch_impl(label, ex, first1, last1, first2, last2, pred_t());
+  return mismatch_exespace_impl(label, ex, first1, last1, first2, last2,
+                                pred_t());
+}
+
+//
+// team impl
+//
+template <class TeamHandleType, class IteratorType1, class IteratorType2,
+          class BinaryPredicateType>
+KOKKOS_FUNCTION ::Kokkos::pair<IteratorType1, IteratorType2> mismatch_team_impl(
+    const TeamHandleType& teamHandle, IteratorType1 first1, IteratorType1 last1,
+    IteratorType2 first2, IteratorType2 last2, BinaryPredicateType predicate) {
+  // checks
+  Impl::static_assert_random_access_and_accessible(teamHandle, first1, first2);
+  Impl::static_assert_iterators_have_matching_difference_type(first1, first2);
+  Impl::expect_valid_range(first1, last1);
+  Impl::expect_valid_range(first2, last2);
+
+  // aliases
+  using return_type          = ::Kokkos::pair<IteratorType1, IteratorType2>;
+  using index_type           = typename IteratorType1::difference_type;
+  using reducer_type         = FirstLoc<index_type>;
+  using reduction_value_type = typename reducer_type::value_type;
+
+  // trivial case: note that this is important,
+  // for OpenMPTarget, omitting special handling of
+  // the trivial case was giving all sorts of strange stuff.
+  const auto num_e1 = last1 - first1;
+  const auto num_e2 = last2 - first2;
+  if (num_e1 == 0 || num_e2 == 0) {
+    return return_type(first1, first2);
+  }
+
+  // run
+  const auto num_elemen_par_reduce = (num_e1 <= num_e2) ? num_e1 : num_e2;
+  reduction_value_type red_result;
+  reducer_type reducer(red_result);
+  ::Kokkos::parallel_reduce(
+      TeamThreadRange(teamHandle, 0, num_elemen_par_reduce),
+      // use CTAD
+      StdMismatchRedFunctor(first1, first2, reducer, std::move(predicate)),
+      reducer);
+
+  teamHandle.team_barrier();
+
+  // decide and return
+  constexpr auto red_min = ::Kokkos::reduction_identity<index_type>::min();
+  if (red_result.min_loc_true == red_min) {
+    // in here means mismatch has not been found
+    if (num_e1 == num_e2) {
+      return return_type(last1, last2);
+    } else if (num_e1 < num_e2) {
+      return return_type(last1, first2 + num_e1);
+    } else {
+      return return_type(first1 + num_e2, last2);
+    }
+  } else {
+    // in here means mismatch has been found
+    return return_type(first1 + red_result.min_loc_true,
+                       first2 + red_result.min_loc_true);
+  }
+}
+
+template <class TeamHandleType, class IteratorType1, class IteratorType2>
+KOKKOS_FUNCTION ::Kokkos::pair<IteratorType1, IteratorType2> mismatch_team_impl(
+    const TeamHandleType& teamHandle, IteratorType1 first1, IteratorType1 last1,
+    IteratorType2 first2, IteratorType2 last2) {
+  using value_type1 = typename IteratorType1::value_type;
+  using value_type2 = typename IteratorType2::value_type;
+  using pred_t      = StdAlgoEqualBinaryPredicate<value_type1, value_type2>;
+  return mismatch_team_impl(teamHandle, first1, last1, first2, last2, pred_t());
 }
 
 }  // namespace Impl
