@@ -53,20 +53,20 @@ void run_single_scenario(const InfoType& scenario_info, int apiId) {
     auto rit =
         KE::move_backward(exespace(), KE::begin(v), KE::end(v), KE::end(v2));
     const int dist = KE::distance(KE::begin(v2), rit);
-    EXPECT_EQ(dist, 5);
+    ASSERT_EQ(dist, 5);
   } else if (apiId == 1) {
     auto rit       = KE::move_backward("mylabel", exespace(), KE::begin(v),
                                  KE::end(v), KE::end(v2));
     const int dist = KE::distance(KE::begin(v2), rit);
-    EXPECT_EQ(dist, 5);
+    ASSERT_EQ(dist, 5);
   } else if (apiId == 2) {
     auto rit       = KE::move_backward(exespace(), v, v2);
     const int dist = KE::distance(KE::begin(v2), rit);
-    EXPECT_EQ(dist, 5);
+    ASSERT_EQ(dist, 5);
   } else if (apiId == 3) {
     auto rit       = KE::move_backward("mylabel", exespace(), v, v2);
     const int dist = KE::distance(KE::begin(v2), rit);
-    EXPECT_EQ(dist, 5);
+    ASSERT_EQ(dist, 5);
   }
 
   // check
