@@ -16,6 +16,7 @@
 
 #include <TestStdAlgorithmsCommon.hpp>
 #include <utility>
+#include <iomanip>
 
 namespace Test {
 namespace stdalgos {
@@ -172,24 +173,15 @@ void verify_data(ViewType1 data_view,  // contains data
       create_mirror_view_and_copy(Kokkos::HostSpace(), test_view_dc);
   if (test_view_h.extent(0) > 0) {
     for (std::size_t i = 0; i < test_view_h.extent(0); ++i) {
-      // std::cout << i << " " << std::setprecision(15) << data_view_h(i) << " "
-      //           << gold_h(i) << " " << test_view_h(i) << " "
-      //           << std::abs(gold_h(i) - test_view_h(i)) << std::endl;
-
       if (std::is_same<gold_view_value_type, int>::value) {
-        EXPECT_EQ(gold_h(i), test_view_h(i));
+        ASSERT_EQ(gold_h(i), test_view_h(i));
       } else {
         const auto error = std::abs(gold_h(i) - test_view_h(i));
-        if (error > 1e-10) {
-          std::cout << i << " " << std::setprecision(15) << data_view_h(i)
-                    << " " << gold_h(i) << " " << test_view_h(i) << " "
-                    << std::abs(gold_h(i) - test_view_h(i)) << std::endl;
-        }
-        EXPECT_LT(error, 1e-10);
+        ASSERT_LT(error, 1e-10) << i << " " << std::setprecision(15) << error
+                                << static_cast<double>(test_view_h(i)) << " "
+                                << static_cast<double>(gold_h(i));
       }
     }
-    // std::cout << " last el: " << test_view_h(test_view_h.extent(0)-1) <<
-    // std::endl;
   }
 }
 
@@ -210,30 +202,11 @@ struct SumBinaryFunctor {
 std::string value_type_to_string(int) { return "int"; }
 std::string value_type_to_string(double) { return "double"; }
 
-template <class Tag, class BopT, class UopT>
-void print_scenario_details(const std::string& name, BopT bop, UopT uop) {
-  (void)bop;
-  (void)uop;
-  std::cout << "transform_inclusive_scan: " << name << ", "
-            << view_tag_to_string(Tag{}) << std::endl;
-}
-
-template <class Tag, class BopT, class UopT, class ValueType>
-void print_scenario_details(const std::string& name, BopT bop, UopT uop,
-                            ValueType init_value) {
-  (void)bop;
-  (void)uop;
-  std::cout << "transform_inclusive_scan: " << name << ", "
-            << view_tag_to_string(Tag{}) << ", "
-            << "init = " << init_value << std::endl;
-}
-
 template <class Tag, class ValueType, class InfoType, class... Args>
 void run_single_scenario(const InfoType& scenario_info,
                          Args... args /* by value on purpose*/) {
   const auto name            = std::get<0>(scenario_info);
   const std::size_t view_ext = std::get<1>(scenario_info);
-  // print_scenario_details<Tag>(name, args...);
 
   auto view_dest =
       create_view<ValueType>(Tag{}, view_ext, "transform_inclusive_scan");
@@ -246,7 +219,7 @@ void run_single_scenario(const InfoType& scenario_info,
     auto r = KE::transform_inclusive_scan(exespace(), KE::cbegin(view_from),
                                           KE::cend(view_from),
                                           KE::begin(view_dest), args...);
-    EXPECT_EQ(r, KE::end(view_dest));
+    ASSERT_EQ(r, KE::end(view_dest));
     verify_data(view_from, view_dest, args...);
   }
 
@@ -255,7 +228,7 @@ void run_single_scenario(const InfoType& scenario_info,
     auto r = KE::transform_inclusive_scan(
         "label", exespace(), KE::cbegin(view_from), KE::cend(view_from),
         KE::begin(view_dest), args...);
-    EXPECT_EQ(r, KE::end(view_dest));
+    ASSERT_EQ(r, KE::end(view_dest));
     verify_data(view_from, view_dest, args...);
   }
 
@@ -263,7 +236,7 @@ void run_single_scenario(const InfoType& scenario_info,
     fill_zero(view_dest);
     auto r =
         KE::transform_inclusive_scan(exespace(), view_from, view_dest, args...);
-    EXPECT_EQ(r, KE::end(view_dest));
+    ASSERT_EQ(r, KE::end(view_dest));
     verify_data(view_from, view_dest, args...);
   }
 
@@ -271,13 +244,70 @@ void run_single_scenario(const InfoType& scenario_info,
     fill_zero(view_dest);
     auto r = KE::transform_inclusive_scan("label", exespace(), view_from,
                                           view_dest, args...);
-    EXPECT_EQ(r, KE::end(view_dest));
+    ASSERT_EQ(r, KE::end(view_dest));
     verify_data(view_from, view_dest, args...);
   }
 
   Kokkos::fence();
 }
 
+template <class Tag, class ValueType, class InfoType, class... Args>
+void run_single_scenario_inplace(const InfoType& scenario_info,
+                                 Args... args /* by value on purpose*/) {
+  const auto name            = std::get<0>(scenario_info);
+  const std::size_t view_ext = std::get<1>(scenario_info);
+
+  // since here we call the in-place operation, we need to use two views:
+  // view1: filled according to scenario and is not modified
+  // view2: filled according scenario and used for the in-place op
+  // Therefore, after the op is done, view_2 should contain the
+  // result of doing exclusive scan.
+  // NOTE: view2 must be filled before every call to the algorithm
+  // because the algorithm acts in place
+
+  auto view_1 = create_view<ValueType>(Tag{}, view_ext,
+                                       "transform_inclusive_scan_view_1");
+  fill_view(view_1, name);
+
+  auto view_2 = create_view<ValueType>(Tag{}, view_ext,
+                                       "transform_inclusive_scan_view_2");
+
+  {
+    fill_view(view_2, name);
+    auto r = KE::transform_inclusive_scan(exespace(), KE::cbegin(view_2),
+                                          KE::cend(view_2), KE::begin(view_2),
+                                          args...);
+    ASSERT_EQ(r, KE::end(view_2));
+    verify_data(view_1, view_2, args...);
+  }
+
+  {
+    fill_view(view_2, name);
+    auto r = KE::transform_inclusive_scan("label", exespace(),
+                                          KE::cbegin(view_2), KE::cend(view_2),
+                                          KE::begin(view_2), args...);
+    ASSERT_EQ(r, KE::end(view_2));
+    verify_data(view_1, view_2, args...);
+  }
+
+  {
+    fill_view(view_2, name);
+    auto r = KE::transform_inclusive_scan(exespace(), view_2, view_2, args...);
+    ASSERT_EQ(r, KE::end(view_2));
+    verify_data(view_1, view_2, args...);
+  }
+
+  {
+    fill_view(view_2, name);
+    auto r = KE::transform_inclusive_scan("label", exespace(), view_2, view_2,
+                                          args...);
+    ASSERT_EQ(r, KE::end(view_2));
+    verify_data(view_1, view_2, args...);
+  }
+
+  Kokkos::fence();
+}
+
 template <class Tag, class ValueType>
 void run_all_scenarios() {
   const std::map<std::string, std::size_t> scenarios = {
@@ -294,18 +324,95 @@ void run_all_scenarios() {
     run_single_scenario<Tag, ValueType>(it, bop_t(), uop_t(), ValueType{2});
     run_single_scenario<Tag, ValueType>(it, bop_t(), uop_t(), ValueType{-1});
     run_single_scenario<Tag, ValueType>(it, bop_t(), uop_t(), ValueType{-2});
+
+    run_single_scenario_inplace<Tag, ValueType>(it, bop_t(), uop_t());
+    run_single_scenario_inplace<Tag, ValueType>(it, bop_t(), uop_t(),
+                                                ValueType{0});
+    run_single_scenario_inplace<Tag, ValueType>(it, bop_t(), uop_t(),
+                                                ValueType{2});
+    run_single_scenario_inplace<Tag, ValueType>(it, bop_t(), uop_t(),
+                                                ValueType{-2});
   }
 }
 
 #if !defined KOKKOS_ENABLE_OPENMPTARGET
 TEST(std_algorithms_numeric_ops_test, transform_inclusive_scan) {
   run_all_scenarios<DynamicTag, double>();
-  // run_all_scenarios<StridedThreeTag, double>();
-  // run_all_scenarios<DynamicTag, int>();
-  // run_all_scenarios<StridedThreeTag, int>();
+  run_all_scenarios<StridedThreeTag, double>();
+  run_all_scenarios<DynamicTag, int>();
+  run_all_scenarios<StridedThreeTag, int>();
 }
 #endif
 
+template <class ValueType>
+struct MultiplyFunctor {
+  KOKKOS_INLINE_FUNCTION
+  ValueType operator()(const ValueType& a, const ValueType& b) const {
+    return (a * b);
+  }
+};
+
+TEST(std_algorithms_numeric_ops_test, transform_inclusive_scan_functor) {
+  using value_type = KE::Impl::ValueWrapperForNoNeutralElement<int>;
+
+  auto test_lambda = [&](auto& functor) {
+    value_type value1;
+    functor.init(value1);
+    ASSERT_EQ(value1.val, 0);
+    ASSERT_EQ(value1.is_initial, true);
+
+    value_type value2;
+    value2.val        = 1;
+    value2.is_initial = false;
+    functor.join(value1, value2);
+    ASSERT_EQ(value1.val, 1);
+    ASSERT_EQ(value1.is_initial, false);
+
+    functor.init(value1);
+    functor.join(value2, value1);
+    ASSERT_EQ(value2.val, 1);
+    ASSERT_EQ(value2.is_initial, false);
+
+    functor.init(value2);
+    functor.join(value2, value1);
+    ASSERT_EQ(value2.val, 0);
+    ASSERT_EQ(value2.is_initial, true);
+
+    value1.val        = 3;
+    value1.is_initial = false;
+    value2.val        = 2;
+    value2.is_initial = false;
+    functor.join(value2, value1);
+    ASSERT_EQ(value2.val, 6);
+    ASSERT_EQ(value2.is_initial, false);
+  };
+
+  int dummy       = 0;
+  using view_type = Kokkos::View<int*, exespace>;
+  view_type dummy_view("dummy_view", 0);
+  using unary_op_type =
+      KE::Impl::StdNumericScanIdentityReferenceUnaryFunctor<int>;
+  {
+    using functor_type =
+        KE::Impl::ExeSpaceTransformInclusiveScanNoInitValueFunctor<
+            exespace, int, int, view_type, view_type, MultiplyFunctor<int>,
+            unary_op_type>;
+    functor_type functor(dummy_view, dummy_view, {}, {});
+
+    test_lambda(functor);
+  }
+
+  {
+    using functor_type =
+        KE::Impl::ExeSpaceTransformInclusiveScanWithInitValueFunctor<
+            exespace, int, int, view_type, view_type, MultiplyFunctor<int>,
+            unary_op_type>;
+    functor_type functor(dummy_view, dummy_view, {}, {}, dummy);
+
+    test_lambda(functor);
+  }
+}
+
 }  // namespace TransformIncScan
 }  // namespace stdalgos
 }  // namespace Test
