@@ -22,9 +22,9 @@
 
 #include <Kokkos_TaskScheduler_fwd.hpp>
 
-#include <impl/Kokkos_TaskQueue.hpp>
-#include <Kokkos_Serial.hpp>
+#include <Serial/Kokkos_Serial.hpp>
 #include <impl/Kokkos_HostThreadTeam.hpp>
+#include <impl/Kokkos_TaskQueue.hpp>
 
 //----------------------------------------------------------------------------
 //----------------------------------------------------------------------------
@@ -121,7 +121,7 @@ class TaskQueueSpecializationConstrained<
     using task_base_type = TaskBase;
     using queue_type     = typename scheduler_type::queue_type;
 
-    task_base_type* const end = (task_base_type*)task_base_type::EndTag;
+    auto* const end = reinterpret_cast<task_base_type*>(task_base_type::EndTag);
 
     execution_space serial_execution_space;
     auto& data = serial_execution_space.impl_internal_space_instance()
@@ -157,7 +157,7 @@ class TaskQueueSpecializationConstrained<
     using task_base_type = TaskBase;
     using queue_type     = typename scheduler_type::queue_type;
 
-    task_base_type* const end = (task_base_type*)task_base_type::EndTag;
+    auto* const end = reinterpret_cast<task_base_type*>(task_base_type::EndTag);
 
     execution_space serial_execution_space;
 
