@@ -166,22 +166,6 @@ TEST(defaultdevicetype, cmd_line_args_device_id) {
   EXPECT_REMAINING_COMMAND_LINE_ARGUMENTS(cla, {"--dummy"});
 }
 
-TEST(defaultdevicetype, cmd_line_args_num_devices) {
-  CmdLineArgsHelper cla = {{
-      "--kokkos-num-devices=5,6",
-      "--kokkos-num-devices=7",
-      "-v",
-  }};
-  Kokkos::InitializationSettings settings;
-  Kokkos::Impl::parse_command_line_arguments(cla.argc(), cla.argv(), settings);
-  EXPECT_TRUE(settings.has_num_devices());
-  EXPECT_EQ(settings.get_num_devices(), 7);
-  // this is the current behavior, not suggesting this cannot be revisited
-  EXPECT_TRUE(settings.has_skip_device()) << "behavior changed see comment";
-  EXPECT_EQ(settings.get_skip_device(), 6) << "behavior changed see comment";
-  EXPECT_REMAINING_COMMAND_LINE_ARGUMENTS(cla, {"-v"});
-}
-
 TEST(defaultdevicetype, cmd_line_args_disable_warning) {
   CmdLineArgsHelper cla = {{
       "--kokkos-disable-warnings=1",
@@ -351,20 +335,6 @@ TEST(defaultdevicetype, env_vars_device_id) {
   EXPECT_EQ(settings.get_device_id(), 33);
 }
 
-TEST(defaultdevicetype, env_vars_num_devices) {
-  EnvVarsHelper ev = {{
-      {"KOKKOS_NUM_DEVICES", "4"},
-      {"KOKKOS_SKIP_DEVICE", "1"},
-  }};
-  SKIP_IF_ENVIRONMENT_VARIABLE_ALREADY_SET(ev);
-  Kokkos::InitializationSettings settings;
-  Kokkos::Impl::parse_environment_variables(settings);
-  EXPECT_TRUE(settings.has_num_devices());
-  EXPECT_EQ(settings.get_num_devices(), 4);
-  EXPECT_TRUE(settings.has_skip_device());
-  EXPECT_EQ(settings.get_skip_device(), 1);
-}
-
 TEST(defaultdevicetype, env_vars_disable_warnings) {
   for (auto const& value_true : {"1", "true", "TRUE", "yEs"}) {
     EnvVarsHelper ev = {{
@@ -420,22 +390,20 @@ TEST(defaultdevicetype, env_vars_tune_internals) {
 }
 
 TEST(defaultdevicetype, visible_devices) {
-#define KOKKOS_TEST_VISIBLE_DEVICES(ENV, CNT, DEV)                    \
-  do {                                                                \
-    EnvVarsHelper ev{ENV};                                            \
-    SKIP_IF_ENVIRONMENT_VARIABLE_ALREADY_SET(ev);                     \
-    Kokkos::InitializationSettings settings;                          \
-    Kokkos::Impl::parse_environment_variables(settings);              \
-    auto computed = Kokkos::Impl::get_visible_devices(settings, CNT); \
-    std::vector<int> expected = DEV;                                  \
-    EXPECT_EQ(expected.size(), computed.size())                       \
-        << ev << "device count: " << CNT;                             \
-    auto n = std::min<int>(expected.size(), computed.size());         \
-    for (int i = 0; i < n; ++i) {                                     \
-      EXPECT_EQ(expected[i], computed[i])                             \
-          << "devices differ at index " << i << '\n'                  \
-          << ev << "device count: " << CNT;                           \
-    }                                                                 \
+#define KOKKOS_TEST_VISIBLE_DEVICES(ENV, CNT, DEV)                      \
+  do {                                                                  \
+    EnvVarsHelper ev{ENV};                                              \
+    SKIP_IF_ENVIRONMENT_VARIABLE_ALREADY_SET(ev);                       \
+    auto computed             = Kokkos::Impl::get_visible_devices(CNT); \
+    std::vector<int> expected = DEV;                                    \
+    EXPECT_EQ(expected.size(), computed.size())                         \
+        << ev << "device count: " << CNT;                               \
+    auto n = std::min<int>(expected.size(), computed.size());           \
+    for (int i = 0; i < n; ++i) {                                       \
+      EXPECT_EQ(expected[i], computed[i])                               \
+          << "devices differ at index " << i << '\n'                    \
+          << ev << "device count: " << CNT;                             \
+    }                                                                   \
   } while (false)
 
 #define DEV(...) \
@@ -444,6 +412,8 @@ TEST(defaultdevicetype, visible_devices) {
 
   // first test with all environment variables that are involved in determining
   // the visible devices so user set var do not mess up the logic below.
+  // KOKKOS_NUM_DEVICES and KOKKOS_SKIP_DEVICE are deprecated since 3.7 and are
+  // not taken into account anymore.
   KOKKOS_TEST_VISIBLE_DEVICES(
       ENV({"KOKKOS_VISIBLE_DEVICES", "2,1"}, {"KOKKOS_NUM_DEVICES", "8"},
           {"KOKKOS_SKIP_DEVICE", "1"}),
@@ -452,10 +422,10 @@ TEST(defaultdevicetype, visible_devices) {
       ENV({"KOKKOS_VISIBLE_DEVICES", "2,1"}, {"KOKKOS_NUM_DEVICES", "8"}, ), 6,
       DEV(2, 1));
   KOKKOS_TEST_VISIBLE_DEVICES(ENV({"KOKKOS_NUM_DEVICES", "3"}), 6,
-                              DEV(0, 1, 2));
+                              DEV(0, 1, 2, 3, 4, 5));
   KOKKOS_TEST_VISIBLE_DEVICES(
       ENV({"KOKKOS_NUM_DEVICES", "4"}, {"KOKKOS_SKIP_DEVICE", "1"}, ), 6,
-      DEV(0, 2, 3));
+      DEV(0, 1, 2, 3, 4, 5));
   KOKKOS_TEST_VISIBLE_DEVICES(ENV({"KOKKOS_VISIBLE_DEVICES", "1,3,4"}), 6,
                               DEV(1, 3, 4));
   KOKKOS_TEST_VISIBLE_DEVICES(
